@@ -33,10 +33,12 @@ import datetime
 import itertools
 import json
 import os
+import sys
 
 import requests
 
-# TODO: Add tests.
+# Byte Order Mark. https://en.wikipedia.org/wiki/Byte_order_mark
+_BOM = u'\ufeff'
 
 
 class Client(object):
@@ -46,7 +48,7 @@ class Client(object):
     retrieve data.
     """
 
-    api_url = 'https://api.emploi-store.fr/api/action'
+    api_url = 'https://api.emploi-store.fr/api'
 
     def __init__(self, client_id=None, client_secret=None):
         if not client_id:
@@ -89,7 +91,7 @@ class Client(object):
     def api_get(self, action, **params):
         """Retrieve JSON information from the API."""
         req = requests.get(
-            self.api_url + action, params=params,
+            self.api_url + '/action' + action, params=params,
             headers={'Authorization': 'Bearer %s' % self.access_token()})
         if req.status_code != 200:
             raise EnvironmentError(
@@ -125,6 +127,47 @@ class Client(object):
         package_json = self.api_get('/package_show', id=package_id)
         return Package(self, **package_json)
 
+    def get_lbb_companies(
+            self, latitude, longitude, distance=10,
+            rome_codes=None, naf_codes=None):
+        """Get a list of hiring companies from La Bonne Boite API.
+
+        Args:
+            latitude: the latitude of the point near which to search for
+                companies.
+            longitude: the longitude of the point near which to search for
+                companies.
+            distance: the maximum distance (in km) to search for companies.
+            rome_codes: a list of ROME IDs defining job groups in which
+                companies should hire.
+            naf_codes: a list of NAF codes defining the activity sector of the
+                companies.
+        Yields:
+            a dict per company, see
+            https://www.emploi-store-dev.fr/portail-developpeur-cms/files/live/sites/emploi-store-dev/files/documents/doc-technique-labonneboite-v1.1.pdf
+            for details of the fields.
+        """
+        params = {
+            'latitude': latitude,
+            'longitude': longitude,
+            'distance': distance,
+        }
+        if rome_codes:
+            params['rome_codes'] = ','.join(rome_codes)
+        if naf_codes:
+            params['naf_codes'] = ','.join(naf_codes)
+        req = requests.get(
+            self.api_url + '/lbb/v1/company/', params=params,
+            headers={'Authorization': 'Bearer %s' % self.access_token()})
+        if req.status_code != 200:
+            raise EnvironmentError(
+                'HTTP error %d for: \n%s' % (req.status_code, req.url))
+        response = req.json()
+        companies = response.get('companies')
+        if companies:
+            for company in companies:
+                yield company
+
 
 class Package(object):
     """A package of resources available.
@@ -182,18 +225,20 @@ class Resource(object):
         self.name = name
         self._id = kwargs.pop('id')
 
-    def _records_batch(self, offset=0, batch_size=200, filters=None):
+    def _records_batch(
+            self, offset=0, batch_size=200, filters=None, fields=None):
         filters_json = json.dumps(filters) if filters else None
+        fields_list = ','.join(fields) if fields else None
         res = self._client.api_get(
             '/datastore_search', limit=batch_size, offset=offset, id=self._id,
-            filters=filters_json)
+            filters=filters_json, fields=fields_list)
         return res.get('records')
 
-    def records(self, batch_size=200, filters=None):
+    def records(self, batch_size=200, filters=None, fields=None):
         """Get all records from resource."""
         offset = 0
         while True:
-            batch = self._records_batch(offset, batch_size, filters)
+            batch = self._records_batch(offset, batch_size, filters, fields)
             for record in batch:
                 yield record
             if len(batch) != batch_size:
@@ -207,13 +252,28 @@ class Resource(object):
         sort them alphabetically.
         """
         records = self.records(batch_size=batch_size)
+        need_utf8_encode = sys.version_info < (3, 0)
         if not fieldnames:
             first = next(records)
             records = itertools.chain([first], records)  # pylint: disable=redefined-variable-type
-            fieldnames = sorted(set(first.keys()) - set(['_id']))
+            keys = set(_strip_bom(k) for k in first.keys())
+            fieldnames = sorted(keys - set(['_id']))
+        if need_utf8_encode:
+            fieldnames = [f.encode('utf-8') for f in fieldnames]
         with open(file_name, 'wt') as csvfile:
             csv_writer = csv.DictWriter(
                 csvfile, fieldnames, extrasaction='ignore')
             csv_writer.writeheader()
             for record in records:
+                record = {_strip_bom(k): v for k, v in record.items()}
+                if need_utf8_encode:
+                    record = {
+                        k.encode('utf-8'): v.encode('utf-8')
+                        for k, v in record.items()}
                 csv_writer.writerow(record)
+
+
+def _strip_bom(field):
+    if field.startswith(_BOM):
+        return field[len(_BOM):]
+    return field
